@@ -1,11 +1,6 @@
 from django import forms
 from django.utils.functional import cached_property
-from wagtail import VERSION as WAGTAIL_VERSION
-
-if WAGTAIL_VERSION >= (3, 0):
-    from wagtail.blocks import FieldBlock, IntegerBlock
-else:
-    from wagtail.core.blocks import FieldBlock, IntegerBlock
+from wagtail.blocks import FieldBlock, IntegerBlock
 
 from wagtailgeowidget import geocoders
 from wagtailgeowidget.helpers import geosgeometry_str_to_struct
@@ -114,6 +109,9 @@ class GoogleMapsBlock(FieldBlock):
             return value
 
         value = geosgeometry_str_to_struct(value)
+        if not value:
+            raise Exception("Error: Cannot parse '{}' into struct".format(value))
+
         value = {
             "lat": value["y"],
             "lng": value["x"],
@@ -123,18 +121,6 @@ class GoogleMapsBlock(FieldBlock):
         return super().to_python(value)
 
 
-class GeoBlock(GoogleMapsBlock):
-    def __init__(self, *args, **kwargs):
-        import warnings
-
-        warnings.warn(
-            "GeoBlock will be deprecated in version 7, use GoogleMapsBlock instead",
-            PendingDeprecationWarning,
-        )
-
-        super().__init__(*args, **kwargs)
-
-
 class LeafletBlock(FieldBlock):
     class Meta:
         icon = "site"
